﻿using TypographyContracts.BindingModels;
using TypographyContracts.ViewModels;
using TypographyDataModels.Enums;
using System.ComponentModel.DataAnnotations;
using TypographyDatabaseImplements.Models;
using System.ComponentModel.DataAnnotations.Schema;
using TypographyDataModels.Models;

namespace TypographyDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; set; }

        [Required]
        public int PrintedId { get; set; }

        [Required]
        public int Count { get; set; }

        [Required]
        public double Sum { get; set; }

        [Required]
        public OrderStatus Status { get; set; }

        [Required]
        public DateTime DateCreate { get; set; }

        public DateTime? DateImplement { get; set; }

        public virtual Printed Printed { get; set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                PrintedId = model.PrintedId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PrintedId = PrintedId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            PrintedName = Printed.PrintedName
        };
    }
}